# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.db import models
from django import get_version
from generic_helpers.utils import ct


class GenericQuerySet(models.query.QuerySet):
    def __init__(self, *args, **kwargs):
        self.ct_field = kwargs.pop('ct_field', 'content_type')
        self.fk_field = kwargs.pop('fk_field', 'object_pk')
        self.gr_field = kwargs.pop('gr_field', 'content_object')
        super(GenericQuerySet, self).__init__(*args, **kwargs)

    def _filter_or_exclude(self, negate, *args, **kwargs):
        if self.gr_field in kwargs:
            instance = kwargs.pop(self.gr_field)
            kwargs.update(**{self.ct_field: ct(instance),
                             self.fk_field: instance.pk
                             })
        return super(GenericQuerySet, self)._filter_or_exclude(negate, *args,
                                                               **kwargs)

    def get_for_object(self, content_object):
        return self.filter(**{self.gr_field: content_object})

    def get_for_model(self, model):
        return self.filter(**{self.gr_field: ct(model)})


class GenericRelationManager(models.Manager):
    def __init__(self, *args, **kwargs):
        self.ct_field = kwargs.pop('ct_field', 'content_type')
        self.fk_field = kwargs.pop('fk_field', 'object_pk')
        self.gr_field = kwargs.pop('gr_field', 'content_object')
        super(GenericRelationManager, self).__init__(*args, **kwargs)

    def get_queryset(self):
        return GenericQuerySet(self.model, ct_field=self.ct_field,
                               fk_field=self.fk_field, gr_field=self.gr_field)

    if get_version() < '1.7':
        def get_query_set(self):
            return self.get_queryset()

    def get_for_object(self, content_object):
        return self.get_queryset().filter(**{
            self.ct_field: ct(content_object),
            self.fk_field: content_object.pk
        })

    def get_for_model(self, model):
        return self.get_queryset().filter(**{self.ct_field: ct(model)})



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.conf import settings


USE_TEXT_OBJECT_PK = getattr(settings, 'GENERIC_HELPERS_USER_TEXT_OBJECT_PK',
                             True)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django import test
from generic_helpers.tests.models import (Story, Picture, Comment, Voting)


class TestBase(test.TestCase):
    def setUp(self):
        self.story_1 = Story.objects.create(text='Story #1')
        self.story_2 = Story.objects.create(text='Story #2')
        self.picture_1 = Picture.objects.create(url='http://example.com/1')
        self.picture_2 = Picture.objects.create(url='http://example.com/2')

    def create_comment_for_story1(self):
        return Comment.objects.create(content_object=self.story_1)

    def create_comment_for_story2(self):
        return Comment.objects.create(content_object=self.story_2)

    def create_comment_for_picture1(self):
        return Comment.objects.create(content_object=self.picture_1)


class TestCreate(TestBase):
    def test_create_comment_via_manager(self):
        comment = self.create_comment_for_story1()
        self.assertEquals(1, Comment.objects.count())
        self.assertEquals(self.story_1, comment.content_object)

    def test_create_comment_via_save(self):
        comment = Comment()
        comment.content_object = self.story_1
        comment.save()
        self.assertEquals(1, Comment.objects.count())
        self.assertEquals(self.story_1, comment.content_object)

    def test_get_for_object(self):
        self.create_comment_for_story1()
        test = Comment.objects.get_for_object
        self.assertEquals(1, test(self.story_1).count())
        self.assertEquals(0, test(self.story_2).count())

    def test_get_for_model(self):
        comment_1 = self.create_comment_for_story1()
        comment_2 = self.create_comment_for_story2()
        comment_3 = self.create_comment_for_picture1()

        self.assertEquals(2, Comment.objects.get_for_model(Story).count())
        self.assertIn(comment_1, Comment.objects.get_for_model(Story))
        self.assertIn(comment_2, Comment.objects.get_for_model(Story))
        self.assertIn(comment_3, Comment.objects.get_for_model(Picture))
        self.assertEquals(1, Comment.objects.get_for_model(Picture).count())

    def test_filter_raw(self):
        comment = self.create_comment_for_story1()
        comment_2 = self.create_comment_for_story2()

        qset = Comment.objects.filter(content_object=self.story_1)
        self.assertEquals(1, qset.count())
        self.assertEquals(comment, qset.get())

        qset = Comment.objects.filter(content_object=self.story_2)
        self.assertEquals(1, qset.count())
        self.assertEquals(comment_2, qset.get())

    def test_custom_grm_content_object_name(self):
        Voting.objects.create(buzz=self.story_1)
        self.assertEquals(1, Voting.objects.filter(buzz=self.story_1).count())

    def test_integer_foreign_key(self):
        with self.settings(GENERIC_HELPERS_USE_TEXT_OBJECT_PK=False):
            pass



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.contrib.contenttypes.models import ContentType


def ct(mdl_cls):
    """
    Shortcut for get_for_model method of ContentType manager
    """
    return ContentType.objects.get_for_model(mdl_cls)



# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from django.utils import six
from generic_helpers.managers import GenericRelationManager
from generic_helpers.settings import USE_TEXT_OBJECT_PK


try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey

# Default value for `related_name` argument
CONTENT_TYPE_RELATED_NAME = 'ct_set_for_%(class)s'


def generic_relation_factory(ct_field='content_type', fk_field='object_pk',
                             gr_field='content_object', manager_attr='objects',
                             ct_related_name=None,
                             class_name='GenericRelationModel',
                             class_name_blank=None, blank=False,
                             fk_field_type=None):
    """
    """
    ct = models.ForeignKey(ContentType,
                           related_name=(ct_related_name or
                                         CONTENT_TYPE_RELATED_NAME),
                           verbose_name=_('content type'),
                           blank=blank, null=blank)

    if fk_field_type is None:
        fk_field_type = models.CharField(_('object ID'), max_length=255,
                                         default='', blank=blank, null=blank)
    fk = fk_field_type

    gr = GenericForeignKey(ct_field=ct_field, fk_field=fk_field)

    class Meta(object):
        abstract = True
        index_together = [(ct_field, fk_field)]

    # If
    if blank:
        class_name = class_name_blank or 'Blank%s' % class_name

    if not six.PY3:
        class_name = six.binary_type(class_name)

    docstring = """
    %(class_name)s

    This class is an abstract model. By inheriting from it, inherited
    model gets two regular fields:

      * %(ct_field)s
      * %(fk_field)s

    And one meta filed: %(gr_field)s

    Thus your model gets new genric relation key

    """ % {'ct_field': ct_field,
           'fk_field': fk_field,
           'gr_field': gr_field,
           'class_name': class_name}

    manager = GenericRelationManager(ct_field=ct_field, fk_field=fk_field,
                                     gr_field=gr_field)

    return type(class_name, (models.Model, ), {ct_field: ct,
                                               fk_field: fk,
                                               gr_field: gr,
                                               manager_attr: manager,
                                               'Meta': Meta,
                                               '__module__': __name__,
                                               '__doc__': docstring
                                               })

if USE_TEXT_OBJECT_PK:
    GenericRelationModel = generic_relation_factory()
    BlankGenericRelationModel = generic_relation_factory(blank=True)
else:
    fk_field_type = models.IntegerField(_('Object ID'), default=0)
    GenericRelationModel = generic_relation_factory(
        fk_field_type=fk_field_type
    )
    BlankGenericRelationModel = generic_relation_factory(
        fk_field_type=fk_field_type,
        blank=True
    )
